@@ -16,6 +16,7 @@ from transformers.utils import logging as hf_logging
 from huggingface_hub import snapshot_download
 from transformers import AutoConfig, BitsAndBytesConfig
 from tqdm import tqdm
+from .state import PipelineState
 
 
 def parse_args(argv: Optional[List[str]] = None) -> argparse.Namespace:
@@ -105,6 +106,22 @@ def parse_args(argv: Optional[List[str]] = None) -> argparse.Namespace:
         action="store_true",
         help="Skip work for outputs that already exist",
     )
+    a.add_argument(
+        "--plan",
+        action="store_true",
+        help="Print a dry-run plan with resource estimates and exit",
+    )
+    a.add_argument(
+        "--resume",
+        action="store_true",
+        help="Resume from .autopack_state.json and skip completed steps",
+    )
+    a.add_argument(
+        "--force-step",
+        nargs="+",
+        default=None,
+        help="One or more step ids to force re-run (e.g., hf:bnb-4bit gguf:Q4_K_M)",
+    )
     a.add_argument(
         "--summary-json",
         action="store_true",
@@ -228,6 +245,22 @@ def parse_args(argv: Optional[List[str]] = None) -> argparse.Namespace:
         action="store_true",
         help="Skip work for outputs that already exist",
     )
+    q.add_argument(
+        "--plan",
+        action="store_true",
+        help="Print a dry-run plan with resource estimates and exit",
+    )
+    q.add_argument(
+        "--resume",
+        action="store_true",
+        help="Resume from .autopack_state.json and skip completed steps",
+    )
+    q.add_argument(
+        "--force-step",
+        nargs="+",
+        default=None,
+        help="One or more step ids to force re-run (e.g., hf gguf:Q4_K_M)",
+    )
 
     # publish command
     p = subparsers.add_parser("publish", help="Publish an exported model folder to the Hugging Face Hub")
@@ -397,6 +430,59 @@ def _human_size(num_bytes: int) -> str:
         size /= 1024.0
 
 
+def _human_time(seconds: float) -> str:
+    minutes = int(seconds // 60)
+    secs = int(seconds % 60)
+    if minutes <= 0:
+        return f"{secs}s"
+    hours = minutes // 60
+    minutes = minutes % 60
+    if hours > 0:
+        return f"{hours}h {minutes}m"
+    return f"{minutes}m {secs}s"
+
+
+def _gpu_total_vram_gb() -> Optional[float]:
+    try:
+        if torch.cuda.is_available():
+            total = 0
+            for i in range(torch.cuda.device_count()):
+                props = torch.cuda.get_device_properties(i)
+                total += getattr(props, "total_memory", 0)
+            if total > 0:
+                return float(total) / (1024**3)
+    except Exception:
+        pass
+    return None
+
+
+def _estimate_step_time_gb(size_gb: float, kind: str) -> float:
+    """Very rough time estimate in seconds based on artifact size and step kind."""
+    base = 60.0
+    per_gb = 45.0
+    if kind.startswith("gguf"):
+        per_gb = 60.0
+    if kind == "onnx":
+        per_gb = 50.0
+    return max(30.0, base + per_gb * max(size_gb, 0.1))
+
+
+def _print_plan(rows: List[Dict[str, object]], totals: Dict[str, float]) -> None:
+    headers = ("Step", "Downloads", "RAM/VRAM", "Temp Disk", "Est Time")
+    print("\nPlanned steps (dry run):\n")
+    print(f"{headers[0]:<20}  {headers[1]:>12}  {headers[2]:>14}  {headers[3]:>12}  {headers[4]:>10}")
+    print("-" * 78)
+    for r in rows:
+        print(
+            f"{r['step']:<20}  {r['download']:>12}  {r['memory']:>14}  {r['temp']:>12}  {r['time']:>10}"
+        )
+    print("-" * 78)
+    print(
+        f"{'TOTAL':<20}  {totals.get('download_h','-'):>12}  {totals.get('memory_h','-'):>14}  {totals.get('temp_h','-'):>12}  {totals.get('time_h','-'):>10}"
+    )
+    print()
+
+
 def _dir_size(path: str) -> int:
     total = 0
     for root, _, files in os.walk(path):
@@ -479,6 +565,98 @@ def run_auto(args: argparse.Namespace) -> int:
 
     variants: List[Tuple[str, dict]] = base_variants
 
+    # Planner and resume state
+    def _plan_for_auto(args_inner: argparse.Namespace, variants_inner: List[Tuple[str, dict]]):
+        try:
+            scan = scan_model(
+                model_id_or_path=args_inner.model,
+                revision=args_inner.revision,
+                trust_remote_code=args_inner.trust_remote_code,
+                local_files_only=True,
+                resolve_cache=False,
+            )
+        except Exception:
+            scan = {"sizes": {"weight_bytes": 0}}
+        weight_bytes = int(((scan or {}).get("sizes") or {}).get("weight_bytes", 0))
+        download_h = _human_size(weight_bytes)
+        vram_total = _gpu_total_vram_gb()
+        from .quantize import _get_model_size_estimate
+        rows = []
+        total_download = float(weight_bytes)
+        total_temp = 0.0
+        total_time = 0.0
+        max_ram = 0.0
+        for name, params in variants_inner:
+            est_gb = float(_get_model_size_estimate(args_inner.model, local_files_only=True, quantization=params["quantization"]))
+            mem_gb = est_gb * 1.5
+            max_ram = max(max_ram, mem_gb)
+            temp_bytes = int(est_gb * (1024**3) * 0.2)
+            total_temp += temp_bytes
+            t = _estimate_step_time_gb(est_gb, f"hf:{name}")
+            total_time += t
+            rows.append({
+                "step": f"hf:{name}",
+                "download": download_h if weight_bytes else "-",
+                "memory": f"~{mem_gb:.1f} GB" + (f" (<= {vram_total:.1f} GB VRAM)" if vram_total else ""),
+                "temp": _human_size(temp_bytes),
+                "time": _human_time(t),
+            })
+        if "onnx" in args_inner.output_format:
+            est_gb = float(_get_model_size_estimate(args_inner.model, local_files_only=True, quantization="none"))
+            mem_gb = est_gb * 1.2
+            max_ram = max(max_ram, mem_gb)
+            temp_bytes = int(est_gb * (1024**3) * 0.1)
+            total_temp += temp_bytes
+            t = _estimate_step_time_gb(est_gb, "onnx")
+            total_time += t
+            rows.append({
+                "step": "onnx",
+                "download": download_h if weight_bytes else "-",
+                "memory": f"~{mem_gb:.1f} GB",
+                "temp": _human_size(temp_bytes),
+                "time": _human_time(t),
+            })
+        if "gguf" in args_inner.output_format:
+            quant_list = args_inner.gguf_quant if args_inner.gguf_quant else ["Q4_K_M", "Q5_K_M", "Q8_0"]
+            for q in [q.upper() for q in quant_list]:
+                est_gb = float(_get_model_size_estimate(args_inner.model, local_files_only=True, quantization="none"))
+                mem_gb = est_gb
+                max_ram = max(max_ram, mem_gb)
+                temp_bytes = int(est_gb * (1024**3) * 0.25)
+                total_temp += temp_bytes
+                t = _estimate_step_time_gb(est_gb, f"gguf:{q}")
+                total_time += t
+                rows.append({
+                    "step": f"gguf:{q}",
+                    "download": download_h if weight_bytes else "-",
+                    "memory": f"~{mem_gb:.1f} GB",
+                    "temp": _human_size(temp_bytes),
+                    "time": _human_time(t),
+                })
+        totals = {
+            "download_h": download_h if weight_bytes else "-",
+            "memory_h": f"~{max_ram:.1f} GB",
+            "temp_h": _human_size(int(total_temp)),
+            "time_h": _human_time(total_time),
+        }
+        _print_plan(rows, totals)
+
+    if getattr(args, "plan", False):
+        _plan_for_auto(args, variants)
+        return 0
+
+    state = PipelineState(PipelineState.default_path(args.output_dir))
+    planned_steps: List[str] = []
+    if "hf" in args.output_format:
+        planned_steps.extend([f"hf:{name}" for name, _ in variants])
+    if "onnx" in args.output_format:
+        planned_steps.append("onnx")
+    if "gguf" in args.output_format:
+        quant_list = args.gguf_quant if args.gguf_quant else ["Q4_K_M", "Q5_K_M", "Q8_0"]
+        planned_steps.extend([f"gguf:{q.upper()}" for q in quant_list])
+    state.init_steps(planned_steps)
+    force_steps = set(getattr(args, "force_step", []) or [])
+
     results = []
     perplexities: Dict[str, float] = {}
 
@@ -512,6 +690,14 @@ def run_auto(args: argparse.Namespace) -> int:
         for name, params in variants:
             out_dir = os.path.join(args.output_dir, name)
             os.makedirs(out_dir, exist_ok=True)
+            step_id = f"hf:{name}"
+            if getattr(args, "resume", False) and state.is_completed(step_id) and step_id not in force_steps:
+                state.mark_skipped(step_id, "already completed")
+                pbar.update_step(f"Skipping {name} (resume)")
+                size_bytes = _dir_size(out_dir)
+                results.append((name, out_dir, size_bytes))
+                pbar.update(1)
+                continue
             
             # Check memory constraints
             if getattr(args, "max_memory_gb", None):
@@ -528,6 +714,7 @@ def run_auto(args: argparse.Namespace) -> int:
             
             # Process variant
             try:
+                state.mark_running(step_id, f"Processing {name}")
                 pbar.update_step(f"Processing {name} variant...")
                 quantize_to_hf(
                     model_id_or_path=source_model,
@@ -543,10 +730,12 @@ def run_auto(args: argparse.Namespace) -> int:
                 size_bytes = _dir_size(out_dir)
                 results.append((name, out_dir, size_bytes))
                 pbar.update_step(f"Completed {name} ({_human_size(size_bytes)})")
+                state.mark_completed(step_id, output_path=out_dir)
             except Exception as e:
                 pbar.update_step(f"Skipping {name} (error: {str(e)[:50]}...)")
                 # Create empty directory entry to maintain progress tracking
                 results.append((name, out_dir, 0))
+                state.mark_failed(step_id, str(e))
             finally:
                 # Force garbage collection between variants
                 gc.collect()
@@ -586,21 +775,30 @@ def run_auto(args: argparse.Namespace) -> int:
         pbar.update_step("Exporting ONNX...")
         onnx_dir = os.path.join(args.output_dir, "onnx")
         os.makedirs(onnx_dir, exist_ok=True)
-        try:
-            if not (getattr(args, "skip_existing", False) and os.path.isdir(onnx_dir) and _dir_size(onnx_dir) > 0):
-                export_onnx(
-                    model_id_or_path=source_model,
-                    output_dir=onnx_dir,
-                    trust_remote_code=args.trust_remote_code,
-                    revision=args.revision,
-                )
+        step_id = "onnx"
+        if getattr(args, "resume", False) and state.is_completed(step_id) and step_id not in force_steps:
+            state.mark_skipped(step_id, "already completed")
             size_bytes = _dir_size(onnx_dir)
             results.append(("onnx", onnx_dir, size_bytes))
-            pbar.update_step(f"Completed ONNX ({_human_size(size_bytes)})")
-        except Exception as e:
-            pbar.update_step(f"Skipping ONNX (error: {str(e)[:50]}...)")
-        finally:
             pbar.update(1)
+        else:
+            try:
+                if not (getattr(args, "skip_existing", False) and os.path.isdir(onnx_dir) and _dir_size(onnx_dir) > 0):
+                    export_onnx(
+                        model_id_or_path=source_model,
+                        output_dir=onnx_dir,
+                        trust_remote_code=args.trust_remote_code,
+                        revision=args.revision,
+                    )
+                size_bytes = _dir_size(onnx_dir)
+                results.append(("onnx", onnx_dir, size_bytes))
+                pbar.update_step(f"Completed ONNX ({_human_size(size_bytes)})")
+                state.mark_completed(step_id, output_path=onnx_dir)
+            except Exception as e:
+                pbar.update_step(f"Skipping ONNX (error: {str(e)[:50]}...)")
+                state.mark_failed(step_id, str(e))
+            finally:
+                pbar.update(1)
 
     # --- Optional GGUF variants for auto (opt-in) ---
     if "gguf" in args.output_format:
@@ -625,6 +823,14 @@ def run_auto(args: argparse.Namespace) -> int:
 
             for quant in quant_list:
                 try:
+                    step_id = f"gguf:{quant}"
+                    if getattr(args, "resume", False) and state.is_completed(step_id) and step_id not in force_steps:
+                        state.mark_skipped(step_id, "already completed")
+                        size_bytes = _dir_size(gguf_out_dir)
+                        results.append((f"gguf-{quant}", gguf_out_dir, size_bytes))
+                        pbar.update(1)
+                        continue
+                    state.mark_running(step_id, f"Exporting GGUF {quant}")
                     pbar.update_step(f"Exporting GGUF {quant}...")
                     expected_file = os.path.join(gguf_out_dir, f"model-{quant}.gguf")
                     if getattr(args, "skip_existing", False) and os.path.isfile(expected_file):
@@ -646,8 +852,10 @@ def run_auto(args: argparse.Namespace) -> int:
                     size_bytes = _dir_size(gguf_out_dir)
                     results.append((f"gguf-{quant}", gguf_out_dir, size_bytes))
                     pbar.update_step(f"Completed GGUF {quant} ({_human_size(size_bytes)})")
+                    state.mark_completed(step_id, output_path=gguf_out_dir)
                 except Exception as e:
                     pbar.update_step(f"Skipping GGUF {quant} (error: {str(e)[:50]}...)")
+                    state.mark_failed(step_id, str(e))
                 finally:
                     pbar.update(1)
         except Exception as e:
@@ -888,12 +1096,115 @@ def run_auto(args: argparse.Namespace) -> int:
 def run_quantize(args: argparse.Namespace) -> int:
     os.makedirs(args.output_dir, exist_ok=True)
     total_steps = ("hf" in args.output_format) + ("onnx" in args.output_format) + ("gguf" in args.output_format)
+    # Planner
+    if getattr(args, "plan", False):
+        # Reuse auto planner with a single HF variant if requested
+        variants: List[Tuple[str, dict]] = []
+        if "hf" in args.output_format:
+            variants.append((args.quantization if args.quantization != "none" else "bf16", {"quantization": args.quantization, "dtype": args.dtype, "device_map": args.device_map}))
+        class _Args:
+            pass
+        _a = _Args()
+        _a.model = args.model
+        _a.revision = args.revision
+        _a.trust_remote_code = args.trust_remote_code
+        _a.output_format = args.output_format
+        _a.gguf_quant = args.gguf_quant
+        # Inline planner from run_auto
+        try:
+            scan = scan_model(
+                model_id_or_path=_a.model,
+                revision=_a.revision,
+                trust_remote_code=_a.trust_remote_code,
+                local_files_only=True,
+                resolve_cache=False,
+            )
+        except Exception:
+            scan = {"sizes": {"weight_bytes": 0}}
+        weight_bytes = int(((scan or {}).get("sizes") or {}).get("weight_bytes", 0))
+        download_h = _human_size(weight_bytes)
+        vram_total = _gpu_total_vram_gb()
+        from .quantize import _get_model_size_estimate
+        rows = []
+        total_temp = 0.0
+        total_time = 0.0
+        max_ram = 0.0
+        for name, params in variants:
+            est_gb = float(_get_model_size_estimate(_a.model, local_files_only=True, quantization=params["quantization"]))
+            mem_gb = est_gb * 1.5
+            max_ram = max(max_ram, mem_gb)
+            temp_bytes = int(est_gb * (1024**3) * 0.2)
+            total_temp += temp_bytes
+            t = _estimate_step_time_gb(est_gb, f"hf:{name}")
+            total_time += t
+            rows.append({
+                "step": f"hf:{name}",
+                "download": download_h if weight_bytes else "-",
+                "memory": f"~{mem_gb:.1f} GB" + (f" (<= {vram_total:.1f} GB VRAM)" if vram_total else ""),
+                "temp": _human_size(temp_bytes),
+                "time": _human_time(t),
+            })
+        if "onnx" in _a.output_format:
+            est_gb = float(_get_model_size_estimate(_a.model, local_files_only=True, quantization="none"))
+            mem_gb = est_gb * 1.2
+            max_ram = max(max_ram, mem_gb)
+            temp_bytes = int(est_gb * (1024**3) * 0.1)
+            total_temp += temp_bytes
+            t = _estimate_step_time_gb(est_gb, "onnx")
+            total_time += t
+            rows.append({
+                "step": "onnx",
+                "download": download_h if weight_bytes else "-",
+                "memory": f"~{mem_gb:.1f} GB",
+                "temp": _human_size(temp_bytes),
+                "time": _human_time(t),
+            })
+        if "gguf" in _a.output_format:
+            q = args.gguf_quant.upper() if isinstance(args.gguf_quant, str) else args.gguf_quant
+            est_gb = float(_get_model_size_estimate(_a.model, local_files_only=True, quantization="none"))
+            mem_gb = est_gb
+            max_ram = max(max_ram, mem_gb)
+            temp_bytes = int(est_gb * (1024**3) * 0.25)
+            total_temp += temp_bytes
+            t = _estimate_step_time_gb(est_gb, f"gguf:{q or 'f16'}")
+            total_time += t
+            rows.append({
+                "step": f"gguf:{q or 'f16'}",
+                "download": download_h if weight_bytes else "-",
+                "memory": f"~{mem_gb:.1f} GB",
+                "temp": _human_size(temp_bytes),
+                "time": _human_time(t),
+            })
+        totals = {
+            "download_h": download_h if weight_bytes else "-",
+            "memory_h": f"~{max_ram:.1f} GB",
+            "temp_h": _human_size(int(total_temp)),
+            "time_h": _human_time(total_time),
+        }
+        _print_plan(rows, totals)
+        return 0
+
     pbar = tqdm(total=total_steps, desc="autopack", unit="step", disable=(total_steps == 0))
+    state = PipelineState(PipelineState.default_path(args.output_dir))
+    planned_steps: List[str] = []
+    if "hf" in args.output_format:
+        planned_steps.append("hf")
+    if "onnx" in args.output_format:
+        planned_steps.append("onnx")
+    if "gguf" in args.output_format:
+        q = args.gguf_quant.upper() if isinstance(args.gguf_quant, str) else args.gguf_quant
+        planned_steps.append(f"gguf:{q}" if q else "gguf")
+    state.init_steps(planned_steps)
+    force_steps = set(getattr(args, "force_step", []) or [])
 
     # Always produce HF format first when requested
     if "hf" in args.output_format:
         pbar.set_description("HF export")
-        if not (getattr(args, "skip_existing", False) and os.path.isdir(args.output_dir) and _dir_size(args.output_dir) > 0):
+        step_id = "hf"
+        if getattr(args, "resume", False) and state.is_completed(step_id) and step_id not in force_steps:
+            state.mark_skipped(step_id, "already completed")
+        elif not (getattr(args, "skip_existing", False) and os.path.isdir(args.output_dir) and _dir_size(args.output_dir) > 0):
+            state.mark_running(step_id, "HF export")
             quantize_to_hf(
                 model_id_or_path=args.model,
                 output_dir=args.output_dir,
@@ -904,6 +1215,7 @@ def run_quantize(args: argparse.Namespace) -> int:
                 revision=args.revision,
                 prune=args.prune,
             )
+            state.mark_completed(step_id, output_path=args.output_dir)
         pbar.update(1)
 
     # Produce ONNX by exporting from the original model id/path (fresh load)
@@ -911,13 +1223,18 @@ def run_quantize(args: argparse.Namespace) -> int:
         pbar.set_description("ONNX export")
         onnx_dir = os.path.join(args.output_dir, "onnx")
         os.makedirs(onnx_dir, exist_ok=True)
-        if not (getattr(args, "skip_existing", False) and os.path.isdir(onnx_dir) and _dir_size(onnx_dir) > 0):
+        step_id = "onnx"
+        if getattr(args, "resume", False) and state.is_completed(step_id) and step_id not in force_steps:
+            state.mark_skipped(step_id, "already completed")
+        elif not (getattr(args, "skip_existing", False) and os.path.isdir(onnx_dir) and _dir_size(onnx_dir) > 0):
+            state.mark_running(step_id, "ONNX export")
             export_onnx(
                 model_id_or_path=args.model,
                 output_dir=onnx_dir,
                 trust_remote_code=args.trust_remote_code,
                 revision=args.revision,
             )
+            state.mark_completed(step_id, output_path=onnx_dir)
         pbar.update(1)
 
     # GGUF export (optional)
@@ -931,7 +1248,11 @@ def run_quantize(args: argparse.Namespace) -> int:
             expected_file = os.path.join(gguf_dir, f"model-{quant_arg}.gguf")
         elif quant_arg is None:
             expected_file = os.path.join(gguf_dir, "model-f16.gguf")
-        if not (getattr(args, "skip_existing", False) and expected_file and os.path.isfile(expected_file)):
+        step_id = f"gguf:{quant_arg}" if isinstance(quant_arg, str) else "gguf"
+        if getattr(args, "resume", False) and state.is_completed(step_id) and step_id not in force_steps:
+            state.mark_skipped(step_id, "already completed")
+        elif not (getattr(args, "skip_existing", False) and expected_file and os.path.isfile(expected_file)):
+            state.mark_running(step_id, "GGUF export")
             export_gguf(
                 model_id_or_path=args.model,
                 output_dir=gguf_dir,
@@ -943,6 +1264,7 @@ def run_quantize(args: argparse.Namespace) -> int:
                 isolate_env=not args.gguf_no_isolation,
                 force=args.gguf_force,
             )
+            state.mark_completed(step_id, output_path=gguf_dir)
         pbar.update(1)
 
     pbar.close()
